# -------------------------------------------------------------------
# matplotlib: Courtney Linder, Philipp Malek
# Python! @ TU Wien                                             
# June 9th, 2017     
# -------------------------------------------------------------------
# Using matplotlib to graph information on video game sales  
# -------------------------------------------------------------------

import matplotlib.pyplot as plt
import numpy as np
import csv

# --------------------------------------

def read_file(file_name):
    """Read graph data from a file.

    Args:
        file_name (str): The name of the file.

    Return:
        list: mylist variable containign read in file

    """
    with open(file_name, 'r',  encoding='utf8') as file:
        reader = csv.reader(file)
        mylist = list(reader)
    return mylist[1:]

def precentage_total_sales(mylist):
    """Extract sales data from mylist, add up totals for each needed
    category, and produce a pie chart.

    Args:
        mylist (list): The list containing the video game sales information.

    """
    # initialize variables to 0
    na,eu,jp,others, total = 0, 0, 0, 0, 0
    
    # for each number in length list, go through and add
    # the float of each position to its respective variable
    for i in range(len(mylist)):
        na += float(mylist[i][6])
        eu += float(mylist[i][7])
        jp += float(mylist[i][8])
        others += float(mylist[i][9])
        total += float(mylist[i][10])
        
    # create the labels for the chart, sizes for each pie piece,
    # explode the North America label, and plot with format 1 decimal
    labels = 'North America','Europe', 'Japan', 'Others'
    sizes = [na/total, eu/total, jp/total, others/total]
    explode = (0.1, 0, 0, 0)

    # create subplots, pie chart, and name the window canvas appropriately 
    fig, ax = plt.subplots()
    fig.canvas.set_window_title("Pie Chart")
    ax.pie(sizes, explode=explode, labels=labels, autopct='%1.1f%%',
            shadow=True, startangle=90)
    ax.set_title("Percentage of Global Sales by Country from 1985-2020")
    
    # equal axis ensures that pie is drawn as a circle
    ax.axis('equal')

def scatter_plot(mylist):
    """Extract sales data from mylist, find all games of 0.2 global sales,
     find the best games above 29.5 sales and plot this on a scatter plot
     with the overall global sales.

    Args:
        mylist (list): The list containing the video game sales information.

    """
    # initialize empty variables
    low_years, low_sales, all_sales, all_years = [], [], [], []
    high_years, high_sales = [], []
    
    # go through mylist ignoring N/A and looking for 0.2 global sales
    for i in range(len(mylist)):
        if mylist[i][3] != 'N/A' and float(mylist[i][10]) == 0.2:
            low_years.append(int(mylist[i][3]))
            low_sales.append(float(mylist[i][10]))
        elif mylist[i][3] != 'N/A' and float(mylist[i][10]) > 29.5:
            high_years.append(int(mylist[i][3]))
            high_sales.append(float(mylist[i][10]))
        elif mylist[i][3] != 'N/A' and mylist[i][10] != 'N/A':
            all_years.append(int(mylist[i][3]))
            all_sales.append(float(mylist[i][10]))

    # create subplots; create scatter plots for 0.2 global sales, sales above
    # 29.5, and the rest of the global sales; name canvas window
    fig, ax = plt.subplots()
    fig.canvas.set_window_title("Scatter Plot")
    ax.scatter(all_years, all_sales, c='blue', marker='.')
    ax.scatter(low_years, low_sales, c='red')
    ax.scatter(high_years, high_sales, c='violet')
    # annotate the highest point with the name of the most popular game
    ax.annotate(mylist[0][1], xy=(high_years[0]-0.5, high_sales[0]-5))

    # name the graph and x and y axis appropriately
    ax.set_title("Global Sales of Games Over the Years\n"
                 "(With a Focus on Global Sales of 0.2 and "
                 "Global Sales Higher than 29.5)")
    ax.set_ylabel("Sales (Number of Copies Sold in Millions)")
    ax.set_xlabel("Years")

def generate_dictionaries(lines):
    """Function to generate needed dictionaries for specific data entries.

    Args:
        lines (list): The list containing the video game sales information.

    Return:
        dictionaries: Sorted dictionaries for platforms, years, genres, and publishers.

    """
    # create empty dictionaries
    platforms, years, genres, publishers = {}, {}, {}, {}
    for line in lines:
        # generate list of platforms
        if line[2] not in platforms:
            platforms[line[2]] = 0
        # generate list of years
        if line[3] not in years:
            years[line[3]] = 0
        # generate list of genres
        if line[4] not in genres:
            genres[line[4]] = 0
        # generate list of publishers
        if line[5] not in publishers:
            publishers[line[5]] = 0

    # increment the values for the categories for each line in the data set
    for line in lines:
        platforms[line[2]] += 1
        years[line[3]] += 1
        genres[line[4]] += 1
        publishers[line[5]] += 1

    # Remove the 'N/A' entry from the list, since it is a string and not an integer/float
    # leaving it in creates problems while parsing
    del years['N/A']

    # Sorting the dicts/list
    sorted_years = sorted(years.items(), key=lambda x: x[1])
    sorted_platforms = sorted(platforms.items(), key=lambda x: x[1])
    sorted_genres = sorted(genres.items(), key=lambda x: x[1])
    sorted_publishers = sorted(publishers.items(), key=lambda x: x[1])

    return sorted_platforms, sorted_years, sorted_genres, sorted_publishers

def creative_method(mylist):
    """Function to use the data in a creative way. In this function, we created
    a figure with four subplots. Each subplot graphs something different.

    Args:
        mylist (list): The list containing the video game sales information.

    """
    # generate sorted dictionaries
    platforms, years, genres, publishers = generate_dictionaries(mylist)
    
    # Create empty arrays of the correct size to store the amount
    # of games sold per category, so we can plot them
    games_per_platform = np.empty(len(platforms))
    games_per_year = np.empty(len(years))
    games_per_genre = np.empty(len(genres))
    games_per_publisher = np.empty(len(publishers))

    # Fill the arrays with the correct values from the lists of tuples
    for i in range(len(platforms)):
        games_per_platform[i] = platforms[i][1]
    for i in range(len(genres)):
        games_per_genre[i] = platforms[i][1]
    for i in range(len(publishers)):
        games_per_publisher[i] = publishers[i][1]
    # create the new window for the graphs, set the canvas window to a title
    fig, per_year = plt.subplots()
    fig.canvas.set_window_title("Subplots")
    #--------------------------------------------------
    # TOP LEFT GRAPH:
    # AMOUNT OF GAMES SOLD PER YEAR, GAMES = Y, YEARS = X
    per_year = plt.subplot(221)

    # Create a list representation of the years and the amount
    # of games from the list of tuples
    yearlist, gamesperyear = [], []
    for i in range(len(years)):
        yearlist.append(years[i][0])
        gamesperyear.append(years[i][1])

    # Markersize 2.5 is important, otherwise the dots will be to big and you can not read anything
    per_year.plot(yearlist, gamesperyear, "o", markersize=2.5)
    per_year.set_title("Sales per Year")
    per_year.set_ylabel("Games Sold")
    # Set x ticks to start at first year in list, end at last year, index every
    # 10 years (for readability)
    per_year.set_xticks(range(int(min(yearlist)), int(max(yearlist)) + 1, 10))
    #--------------------------------------------------
    # TOP RIGHT GRAPH:
    # Plot the amount of games sold for the top 5 publishers,
    # the publishers should be x-axis and games y-axis
    top5publishers = []
    for publisher in publishers[-6:]:
        # needed to shorten the length of publishers that are too long
        # instead just use the first letter of that company's name
        if len(publisher[0]) > 12:
            top5publishers.append(publisher[0][0])
        else:
            top5publishers.append(publisher[0])
    # create the subplot with the axis and labels
    per_publisher = plt.subplot(222)
    per_publisher.plot(range(0, 5), games_per_publisher[-5:], "o")
    per_publisher.set_title("Sales per Publisher")
    per_publisher.set_ylabel("Games Sold")
    per_publisher.set_xticklabels(top5publishers, rotation='45', size='smaller')
    #--------------------------------------------------
    # BOTTOM LEFT GRAPH
    # Plot the amount of games sold for the top 5 genres,
    # the genres should be x-axis and games y-axis
    top5genres = []
    for genre in genres[-6:]:
        top5genres.append(genre[0])
    # create the next subplot with the axis and labels
    per_genre = plt.subplot(223)
    per_genre.plot(range(0, 5), games_per_genre[-5:], "o")
    per_genre.set_title("Sales per Genre")
    per_genre.set_ylabel("Games Sold")
    per_genre.set_xticks(range(0, 5))
    per_genre.set_xticklabels(top5genres, rotation='45')
    #--------------------------------------------------
    # BOTTOM RIGHT GRAPH
    # Plot the amount of games sold for the top 5 platforms,
    # the platforms should be x-axis and games y-axis
    top5platforms = []
    for platform in platforms[-6:]:
        top5platforms.append(platform[0])
    # create the last subplot with axis and labels
    per_platform = plt.subplot(224)
    per_platform.plot(range(0, 5), games_per_platform[-5:], "o")
    per_platform.set_title("Sales per Platform")
    per_platform.set_ylabel("Games Sold")
    per_platform.set_xticks(range(0, 5))
    per_platform.set_xticklabels(top5platforms, rotation='45')

    # ensure that the subplots don't overlap
    plt.tight_layout()
    
def main():
    """Reads in data on video games sales from 1985-2020 and
    displays it various graphs."""
    
    mylist = read_file("vgsales.csv")
    precentage_total_sales(mylist)
    scatter_plot(mylist)
    creative_method(mylist)
    plt.show()
    
if __name__ == "__main__":
    main()
